@@ -1,9 +1,10 @@
 use std::io::Write;
 
+use ruff_db::diagnostic::Diagnostic;
 use ruff_source_file::LineColumn;
 
 use crate::fs::relativize_path;
-use crate::message::{Emitter, EmitterContext, OldDiagnostic};
+use crate::message::{Emitter, EmitterContext};
 
 /// Generate error workflow command in GitHub Actions format.
 /// See: [GitHub documentation](https://docs.github.com/en/actions/reference/workflow-commands-for-github-actions#setting-an-error-message)
@@ -14,12 +15,13 @@ impl Emitter for GithubEmitter {
     fn emit(
         &mut self,
         writer: &mut dyn Write,
-        diagnostics: &[OldDiagnostic],
+        diagnostics: &[Diagnostic],
         context: &EmitterContext,
     ) -> anyhow::Result<()> {
         for diagnostic in diagnostics {
-            let source_location = diagnostic.compute_start_location();
-            let location = if context.is_notebook(&diagnostic.filename()) {
+            let source_location = diagnostic.expect_ruff_start_location();
+            let filename = diagnostic.expect_ruff_filename();
+            let location = if context.is_notebook(&filename) {
                 // We can't give a reasonable location for the structured formats,
                 // so we show one that's clearly a fallback
                 LineColumn::default()
@@ -27,15 +29,13 @@ impl Emitter for GithubEmitter {
                 source_location
             };
 
-            let end_location = diagnostic.compute_end_location();
+            let end_location = diagnostic.expect_ruff_end_location();
 
             write!(
                 writer,
-                "::error title=Ruff{code},file={file},line={row},col={column},endLine={end_row},endColumn={end_column}::",
-                code = diagnostic
-                    .noqa_code()
-                    .map_or_else(String::new, |code| format!(" ({code})")),
-                file = diagnostic.filename(),
+                "::error title=Ruff ({code}),file={file},line={row},col={column},endLine={end_row},endColumn={end_column}::",
+                code = diagnostic.secondary_code_or_id(),
+                file = filename,
                 row = source_location.line,
                 column = source_location.column,
                 end_row = end_location.line,
@@ -45,13 +45,15 @@ impl Emitter for GithubEmitter {
             write!(
                 writer,
                 "{path}:{row}:{column}:",
-                path = relativize_path(&*diagnostic.filename()),
+                path = relativize_path(&filename),
                 row = location.line,
                 column = location.column,
             )?;
 
-            if let Some(code) = diagnostic.noqa_code() {
+            if let Some(code) = diagnostic.secondary_code() {
                 write!(writer, " {code}")?;
+            } else {
+                write!(writer, " {id}:", id = diagnostic.id())?;
             }
 
             writeln!(writer, " {}", diagnostic.body())?;
