@@ -2,6 +2,7 @@ pub(crate) use async_busy_wait::*;
 pub(crate) use async_function_with_timeout::*;
 pub(crate) use async_zero_sleep::*;
 pub(crate) use blocking_http_call::*;
+pub(crate) use blocking_http_call_httpx::*;
 pub(crate) use blocking_open_call::*;
 pub(crate) use blocking_process_invocation::*;
 pub(crate) use blocking_sleep::*;
@@ -13,6 +14,7 @@ mod async_busy_wait;
 mod async_function_with_timeout;
 mod async_zero_sleep;
 mod blocking_http_call;
+mod blocking_http_call_httpx;
 mod blocking_open_call;
 mod blocking_process_invocation;
 mod blocking_sleep;
