@@ -1,12 +1,12 @@
-use ruff_python_ast::{Expr, ExprTuple};
+use ruff_python_ast::{Expr, Stmt};
 
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_semantic::analyze::typing::is_dict;
 use ruff_python_semantic::{Binding, SemanticModel};
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
+use crate::{AlwaysFixableViolation, Edit, Fix};
 
 /// ## What it does
 /// Checks for dictionary unpacking in a for loop without calling `.items()`.
@@ -34,30 +34,47 @@ use crate::checkers::ast::Checker;
 /// for city, population in data.items():
 ///     print(f"{city} has population {population}.")
 /// ```
-#[violation]
-pub struct DictIterMissingItems;
+///
+/// ## Known problems
+/// If the dictionary key is a tuple, e.g.:
+///
+/// ```python
+/// d = {(1, 2): 3, (3, 4): 5}
+/// for x, y in d:
+///     print(x, y)
+/// ```
+///
+/// The tuple key is unpacked into `x` and `y` instead of the key and values. This means that
+/// the suggested fix of using `d.items()` would result in different runtime behavior. Ruff
+/// cannot consistently infer the type of a dictionary's keys.
+///
+/// ## Fix safety
+/// Due to the known problem with tuple keys, this fix is unsafe.
+#[derive(ViolationMetadata)]
+pub(crate) struct DictIterMissingItems;
 
 impl AlwaysFixableViolation for DictIterMissingItems {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Unpacking a dictionary in iteration without calling `.items()`")
+        "Unpacking a dictionary in iteration without calling `.items()`".to_string()
     }
 
     fn fix_title(&self) -> String {
-        format!("Add a call to `.items()`")
+        "Add a call to `.items()`".to_string()
     }
 }
 
-pub(crate) fn dict_iter_missing_items(checker: &mut Checker, target: &Expr, iter: &Expr) {
-    let Expr::Tuple(ExprTuple { elts, .. }) = target else {
+/// PLE1141
+pub(crate) fn dict_iter_missing_items(checker: &Checker, target: &Expr, iter: &Expr) {
+    let Expr::Tuple(tuple) = target else {
         return;
     };
 
-    if elts.len() != 2 {
+    if tuple.len() != 2 {
         return;
-    };
+    }
 
-    let Some(name) = iter.as_name_expr() else {
+    let Expr::Name(name) = iter else {
         return;
     };
 
@@ -73,38 +90,32 @@ pub(crate) fn dict_iter_missing_items(checker: &mut Checker, target: &Expr, iter
     }
 
     // If we can reliably determine that a dictionary has keys that are tuples of two we don't warn
-    if is_dict_key_tuple_with_two_elements(checker.semantic(), binding) {
+    if is_dict_key_tuple_with_two_elements(binding, checker.semantic()) {
         return;
     }
 
-    let mut diagnostic = Diagnostic::new(DictIterMissingItems, iter.range());
-    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+    let mut diagnostic = checker.report_diagnostic(DictIterMissingItems, iter.range());
+    diagnostic.set_fix(Fix::unsafe_edit(Edit::range_replacement(
         format!("{}.items()", name.id),
         iter.range(),
     )));
-    checker.diagnostics.push(diagnostic);
 }
 
 /// Returns true if the binding is a dictionary where each key is a tuple with two elements.
-fn is_dict_key_tuple_with_two_elements(semantic: &SemanticModel, binding: &Binding) -> bool {
+fn is_dict_key_tuple_with_two_elements(binding: &Binding, semantic: &SemanticModel) -> bool {
     let Some(statement) = binding.statement(semantic) else {
         return false;
     };
 
-    let Some(assign_stmt) = statement.as_assign_stmt() else {
+    let Stmt::Assign(assign_stmt) = statement else {
         return false;
     };
 
-    let Some(dict_expr) = assign_stmt.value.as_dict_expr() else {
+    let Expr::Dict(dict_expr) = &*assign_stmt.value else {
         return false;
     };
 
-    dict_expr.iter_keys().all(|elt| {
-        elt.is_some_and(|x| {
-            if let Expr::Tuple(ExprTuple { elts, .. }) = x {
-                return elts.len() == 2;
-            }
-            false
-        })
-    })
+    dict_expr
+        .iter_keys()
+        .all(|key| matches!(key, Some(Expr::Tuple(tuple)) if tuple.len() == 2))
 }
