@@ -1,9 +1,9 @@
 use ruff_python_ast::Expr;
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 use crate::rules::pylint::helpers::in_dunder_method;
 
@@ -28,21 +28,19 @@ use crate::rules::pylint::helpers::in_dunder_method;
 ///
 /// ## References
 /// - [CodeQL: `py-init-method-is-generator`](https://codeql.github.com/codeql-query-help/python/py-init-method-is-generator/)
-#[violation]
-pub struct YieldInInit;
+#[derive(ViolationMetadata)]
+pub(crate) struct YieldInInit;
 
 impl Violation for YieldInInit {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`__init__` method is a generator")
+        "`__init__` method is a generator".to_string()
     }
 }
 
 /// PLE0100
-pub(crate) fn yield_in_init(checker: &mut Checker, expr: &Expr) {
-    if in_dunder_method("__init__", checker.semantic(), checker.settings) {
-        checker
-            .diagnostics
-            .push(Diagnostic::new(YieldInInit, expr.range()));
+pub(crate) fn yield_in_init(checker: &Checker, expr: &Expr) {
+    if in_dunder_method("__init__", checker.semantic(), checker.settings()) {
+        checker.report_diagnostic(YieldInInit, expr.range());
     }
 }
